#!/usr/bin/ruby
#
# (c) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

class GPSNmea

  def initialize check=true, debug=false
    @check = check
    @debug = debug
    @sata = nil
  end

  def nmeachk s
    x=0
    chk=s[-2..-1].hex
    s[1..-1].each_byte do |c|
      break if c == ?*
      x ^= c
    end
   (x == chk)
  end

  def decode msg
    msg.chomp!
    return nil if @check and !nmeachk(msg)
    case msg
      when /^\$GPRMC/
      decode_rmc msg
      when /^\$GPGGA/
      decode_gga msg
      when /^\$GPGSA/
      decode_gsa msg
      when /^\$GPGSV/
      decode_gsv msg
    end
  end

  def dec_time ts
    us=nil
    t=Time.now.gmtime
    h=ts[0..1].to_i
    m=ts[2..3].to_i
    is=ts[4..5].to_i
    s=ts[4..-1].to_f
    us= (1000000.0*(s-is)).to_i
    Time.utc(t.year,t.month,t.day,h,m,is,us)
  end

  def decode_gga msg
    t={:type=>:gga}
    begin
      args = msg.split(/[,*]/)
      t[:utc] = dec_time args[1]
      t[:lat],t[:alat] = decpos args[2],args[3]
      t[:lon],t[:alon] = decpos args[4],args[5]
      t[:status] = case args[6]
		   when '0'
		     'Invalid'
		   when '1'
		     'GPS'
		   when '2'
		     'DPGS'
		   end
      t[:nsat] = args[7].to_i
      t[:hdop] = args[8].to_f
      t[:alt] = "#{args[9]} #{args[10].downcase}"
      t[:ht] = "#{args[11]} #{args[12].downcase}"
      puts t.inspect if @debug
    rescue
      t=nil
    end
    t
  end
  
  def decode_gsa msg
    @prn =[]
    t={:type=>:gsa}
    args = msg.split(/[,*]/)
    t[:mode] = args[1] == 'M' ? 'Manual' : 'Auto'
    t[:mode] << case args[2]
		when '1'
		  ' No fix'
		when '2'
		  ' 2D'
		when '3'
		  ' 3D'
		else
		  ''
		end
    3.upto(14) {|i| @prn << args[i] unless (args[i].nil? or args[i].empty?)}
    t[:pdop] = args[15]
    t[:hdop] = args[16]
    t[:vdop] = args[17]
    t[:prn] = @prn
    puts t.inspect if @debug
    t
  end

  def decode_gsv msg
    t = nil
    args = msg.split(/[,*]/)
    if args[2] == '1'
      @sata = []
    end
    nelem = (args.size-5)
    4.step(nelem, 4) do |n|
      if args[n].size and @sata
	p = args[n]
	e = args[n+1].to_i
	a = args[n+2].to_i
	s = args[n+3].empty? ? nil : args[n+3].to_i
	u = @prn && @prn.include?(p)
	@sata << [p,e,a,s,u] 
      end
    end
    if ((args[1] == args[2]) and @sata)
      t = {:type => :gsv}
      t[:sats] = @sata
      puts t.inspect if @debug
    end
    t
  end

  def decutc ts,ds
    Time.utc(2000+ds[4..5].to_i,ds[2..3].to_i,ds[0..1].to_i,
	       ts[0..1],ts[2..3],ts[4..5])
  end

  def decpos pos,ind
    sign=(ind == 'S' or ind == 'W')
    fmt=nil
    case ind 
    when 'N','S'
      dd = pos[0..1].to_i
      dm = pos[2..-1].to_f
      fmt = "%02d:%02d:%05.2f#{ind}"
    when 'E','W'
      dd = pos[0..2].to_i
      dm = pos[3..-1].to_f
      fmt = "%03d:%02d:%05.2f#{ind}"
    end
    im = dm.to_i
    ds = 60.0*(dm - im)
    fpos = dd+dm/60.0;
    fpos = -fpos if sign
    spos = fmt % [dd,im,ds]
    [fpos,spos]
  end

  def decode_rmc msg
    t={:type=>:rmc}
    begin
      args = msg.split(/[,*]/)
      t[:lat],t[:alat] = decpos args[3],args[4]
      t[:lon],t[:alon] = decpos args[5],args[6]
      t[:utc] = decutc args[1],args[9]
      t[:valid] = (args[2] == 'A')
      t[:spd] = args[7].to_f
      t[:cse] = args[8].to_f
      t[:var] = args[10].to_f
      t[:var] *=-1 if args[11] == 'E'
      puts t.inspect if @debug
    rescue
      t=nil
    end
    t
  end
end

if __FILE__ == $0
  nm = GPSNmea.new true,true
  ARGF.each do |l|
    nm.decode l
  end
end
