#!/usr/bin/ruby 

#
# (c) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

require 'optparse'
require "serialport"

class WBT201

  MODES=[nil, 'Log Mode','Low speed Limit','High speed Limit',
    'Heading Interval', 'Low Speed','Middle Speed',
    'High Speed',nil,'Speed mode time','Low-Middle interval',
    'Middle-High interval','High-High interval',
    'Time Mode interval', 'Distance Mode interval']

  LOGS=['None','Heading Change','Speed Change','Time Interval',
    'Distance Interval','Mix (T & D)']

  def initialize dev=0
    @verbose=@dump=nil
    ARGV.options do |opt|
      opt.banner = "Usage: wbt201.rb [options]"
      opt.on('-s','--serial-port SERIALPORT') {|o| dev=o}
      opt.on('-d','--dump-gps','false') {@dump=true}
      opt.on('-v','--verbose','false') {@verbose=true}
      opt.on('-h', "--help", "Show this message") {puts opt; exit}
      begin
	opt.parse!
      rescue
	puts opt ; exit
      end
    end
    if @verbose
      @verbose=File.open("WBT201_#{Time.now.to_i}.log","wb")
      @verbose.sync = true
      at_exit{@verbose.close}
    end
    STDERR.puts "Dev #{dev}" if @verbose
    @ser = SerialPort.new(dev,57600)
    @ser.read_timeout=1000
  end


  def timeout n
    @ser.read_timeout=n
  end

  def setup
    STDOUT.sync = true
    l=nil
    done = false
    1.upto(5) do |m|
      sendcmd
      0.upto(10) do |n|
	l=@ser.gets
	@verbose.puts("<< #{l}") if @verbose
	if (l and l.match(/^@AL/))
	  done =true
	  break
	end
      end
      break if done
    end
    abort 'Init fails' if l.nil? or !l.match('LoginOK')
    @ser.read_timeout=250
  end

  def getdata c
    ll=nil
    while l=@ser.gets
      ll=l
      @verbose.puts("<< #{ll}") if @verbose
      break if ll.match(/^#{c}/)
    end
    ll
  end

  def dumpdata
    loop do
      l=@ser.gets
      puts(l) if l
    end
  end

  def sendcmd q=nil
    cmd="@AL"
    cmd << ",#{q}" if q
    @verbose.puts(">> #{cmd}") if @verbose
    @ser.print "#{cmd}\r\n"
    cmd
  end

  def query q
    c = sendcmd q
    r = getdata c
    r ? r.gsub("#{c},",'') : nil
  end
  
  def explore
    puts "Name : #{query('7,1')}"
    puts "Info : #{query('7,2')}"
    puts "Hardware Version: #{query('8,1')}"
    puts "Software Version: #{query('8,2')}"
    puts "Log Version: #{query('8,3')}"
    lm=query("6,1").to_i
    puts "#{MODES[1]}: #{LOGS[lm]} (#{lm})"    
    2.upto(14) do |n|
      next unless MODES[n]
      puts "#{MODES[n]}: #{query("6,#{n}")}"
    end
    puts "Log Start Addr: #{query('5,1')}"
    puts "Log End Addr: #{query('5,2')}"
    puts "Log Start Area: #{query('5,9')}"
    puts "Log End Area: #{query('5,10')}"
    
    puts "AutoSleep: #{query('1,1')}"
    puts "OverSpeed: #{query('1,2')}"

#    timeout 1000
#    In Windows .exe, not known
#    %w/3,1 4,1 5,4 7,3/.each do |cs|
#      puts "@AL,#{cs}: #{query(cs)}"
#    end

    if @dump
      sendcmd '2,1'
      dumpdata
    end
  end
end

if __FILE__ == $0
  m=WBT201.new
  m.setup
  m.explore
end
