#!/usr/bin/env ruby
#
# (c) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

require 'libglade2'
require 'wbt201-ui'
require "serialport"
require 'nmea'
require 'optparse'

class Wbt201Glade

  XC = 0.5
  YC = 0.6
  TWOPI = 2 *Math::PI
  RADIUS = 0.48
  BHT = 1.8

  WMODES=[nil, nil, 'low_speed_limit', 'high_speed_limit', 'heading_mode_int',
    'low_speed', 'middle_speed', 'high_speed', nil, 'speed_mode_time',
    'low_middle_int', 'middle_high_int', 'high_high_int',
    'time_mode_int', 'dist_mode_int']
  WSETS= [nil, 'auto_sleep', 'over_speed']

  def on_dev_type_changed(widget)
    @dev = @glade.get_widget("devcombo").active_iter.get_value(0)
  end

  def disconn
    begin
      @ser.print("@AL,2,1\r\n")
      @chan.close if @chan
      @ser.close if @ser
    rescue
    ensure
      @ser = @chan = nil
      @glade.get_widget("conn_state").stock = Gtk::Stock::NO
      @glade.get_widget("conn_button").label = 'Connect'       
      @constate=false
    end
  end

  def setup
    @nm = GPSNmea.new
    @chan = nil
    @id = nil
    @constate = false
    @actions = []
    @glade.get_widget("devcombo").set_active(0)
    @glade.get_widget("logcombo").set_active(0)
    @vals={}
    @t = nil
    @tv = @glade.get_widget("gps_text")
    @tb = @tv.buffer
    @tm = @tb.create_mark('eob',@tb.end_iter, false)
    @da = @glade.get_widget("drawingarea1")
    at_exit { disconn }
    @verbose=@dump=nil
    @gps={}
    ARGV.options do |opt|
      opt.banner = "Usage: wbt201.rb [options]"
      opt.on('-d','--dump-gps','false') {@dump=true}
      opt.on('-v','--verbose','false') {@verbose=true}
      opt.on('-h', "--help", "Show this message") {puts opt; exit}
      begin
        opt.parse!
      rescue
        puts opt ; exit
      end
    end
    if @verbose
      @verbose=File.open("CMD-WBT201.log","w")
      @verbose.sync = true
      at_exit{@verbose.close}
    end
    if @dump
      @dump=File.open("GPS-WBT201.log","w")
      @dump.sync = true
      at_exit{@dump.close}
    end
  end

  def set_colour d
    cols = case d[4]
           when false
             [1,1,1]
           when true
             d[3] < 25 ? [0.2,0.2,0.8] : [0,0,1] 
           else
             [0.5,0.5,0.5]
           end
    @cr.set_source_rgb(*cols)
  end

  def draw
    @cr.select_font_face("Sans", 
                         Cairo::FONT_SLANT_NORMAL, Cairo::FONT_WEIGHT_NORMAL)
    @cr.set_font_size(0.05)
    @cr.set_line_width(0.01)
    @cr.arc(XC, YC, RADIUS, 0, TWOPI)
    @cr.stroke
    @cr.set_source_rgb(0.2, 0.2, 0.2)
    @cr.set_line_width(0.005)
    @cr.arc(XC, YC, 0.32, 0, TWOPI)
    @cr.stroke
    @cr.arc(XC, YC, 0.16, 0, TWOPI)
    @cr.stroke
    0.upto(7) do |i|
      rad =  i * (Math::PI / 8)
      x1 = XC + RADIUS * Math.sin(rad)
      y1 = YC + RADIUS * Math.cos(rad)
      rad += Math::PI
      x2 = XC + RADIUS * Math.sin(rad)
      y2 = YC + RADIUS * Math.cos(rad)
      @cr.move_to(x1,y1)
      @cr.line_to(x2,y2)
      @cr.stroke
    end
    if @t
      @t[:sats].each do |d|
	rad = d[2] * (Math::PI / 180)
	rng = (90 - d[1]) * (RADIUS/90.0)
	x1 = XC + rng*Math.sin(rad)
	y1 = YC - rng*Math.cos(rad)
	set_colour d
	@cr.arc(x1, y1, 0.06, 0, TWOPI)
	@cr.fill
	@cr.stroke
	@cr.set_line_width(0.0025)
	@cr.set_source_rgb(0, 0, 0)
	@cr.arc(x1, y1, 0.06, 0, TWOPI)
	@cr.stroke
	@cr.move_to(x1-0.03, y1+0.02)
	@cr.set_source_rgb(1, 1, 1) if d[3] and d[4]
	@cr.show_text(d[0])
	@cr.stroke
      end

      ns = @t[:sats].size
      sw = 0.9 / ns 
      sw = 0.1 if sw > 0.1
      n = 0
      @t[:sats].each do |d|
	x = 0.05 + n * sw + 0.01
	w = sw - 0.02
	
	ht = (d[3]) ? d[3] /100.0 : 0
	
	set_colour d
	@cr.rectangle(x, BHT - ht , w, ht)
	@cr.fill
	@cr.stroke
	
	@cr.set_line_width(0.01)
	@cr.set_source_rgb(0, 0, 0)
	@cr.rectangle(x, BHT - ht , w, ht)
	@cr.stroke
	
	@cr.move_to(x-0.01, BHT + 0.1)
	@cr.show_text(d[0])
	@cr.stroke

	if d[3]
	  @cr.move_to(x-0.01, BHT-0.02-ht)
	  @cr.show_text(d[3].to_s)
	  @cr.stroke
	end
	n += 1
      end
    end
    @t = nil
  end

  def on_draw_expose_event widget,event
    @cr = widget.window.create_cairo_context
    sf = widget.window.size[0]
    sf = widget.window.size[1] if widget.window.size[1] < sf
    @cr.scale(sf,sf)
    @cr.set_line_width(0.04)
      
    @cr.save do
      @cr.set_source_color(Gdk::Color.new(65535, 65535, 65535))
      @cr.gdk_rectangle(Gdk::Rectangle.new(0, 0, 1, 2))
      @cr.fill
    end
    draw
  end
  
  def on_connect_clicked(widget)
    if @constate
      disconn
    else
      begin
      @ser = SerialPort.new(@dev,57600)
      @chan = GLib::IOChannel.new(@ser.fileno) 
	@id = @chan.add_watch(GLib::IOChannel::IN) {|channel, condition| do_chan(channel,condition) }
	@actions = [nil, nil, nil, nil, nil]
	kick_actions
      rescue
      end
    end
  end

  def do_chan(chan, cond)
    rv = false
    l=nil
    begin
      l = @chan.readline
      rv = true
    rescue
      disconn
    end
    process_line(l) if l
    rv
  end

  def kick_actions
    if @actions.size > 0
      act = @actions.shift
      cmd = '@AL'
      cmd << ",#{act}" if act
      @verbose.puts("Sending #{cmd}") if @verbose
      @ser.print "#{cmd}\r\n"
    end
  end

  def process_line l
    case l
    when /^\$G/
      process_nmea l
    when /^@AL/
      process_al_resp l
    end
    kick_actions
  end

  def process_nmea l
    @dump.puts(l) if @dump    
    if (t = @nm.decode l)
      if @tb.line_count > 1024
	@tb.delete(@tb.start_iter,@tb.get_iter_at_line(256))
      end
      @tb.insert(@tb.end_iter, "#{l}\n")
      @tv.scroll_to_iter(@tb.end_iter, 0.0,false, 0.0, 0.5)
      case t[:type]
      when :rmc
	@glade.get_widget("utc").text = t[:utc].to_s
	@glade.get_widget("lat").text = t[:alat]
	@glade.get_widget("lon").text = t[:alon]
	@glade.get_widget("speed").text = t[:spd].to_s
	@glade.get_widget("course").text = t[:cse].to_s
      when :gga
	@glade.get_widget("utc").text = t[:utc].to_s
	@glade.get_widget("lat").text = t[:alat]
	@glade.get_widget("lon").text = t[:alon]
	@glade.get_widget("alt").text = t[:alt]	
	@gps[:nsat] = t[:nsat]
	@gps[:hdop] = t[:hdop]
	@gps[:status] = t[:status]
      when :gsa
	@gps[:mode] = t[:mode]
	@gps[:hdop] = t[:hdop]
	@gps[:vdop] = t[:vdop]
	@gps[:pdop] = t[:pdop]
      when :gsv
	if ((@gps[:tsat] = t[:sats].size) > 0)
	  if @t.nil?
	    @t = t.dup
	    @da.queue_draw_area(0, 0, *@da.window.size)
	  end
	end
      end

      @glade.get_widget("status").text = "#{@gps[:status]} #{@gps[:mode]}"	
      @glade.get_widget("tiff").text = 
	"Sats: #{@gps[:nsat]} / #{@gps[:tsat]} H/V/PDOP: #{@gps[:hdop]}/#{@gps[:vdop]}/#{@gps[:pdop]}"
    end
  end

  def process_al_resp l
    l.chomp!
    @verbose.puts(l) if @verbose
    case l
    when /^@AL,LoginOK/
      unless @constate
	@glade.get_widget("conn_state").stock = Gtk::Stock::YES
	@glade.get_widget("conn_button").label = 'Disconnect'       
	@constate=true
	@actions = %w(7,1 7,2 8,1 8,2 8,3 5,1 5,2 1,1 1,2 6,1 6,2 6,3 6,4 6,5 6,6 6,7 6,9 6,10 6,11 6,12 6,13 6,14) 
      else
	@actions.shift while @actions.size > 0 and @actions.first.nil?	
      end
    when /^@AL,1,1,(\d+)/
      val=$1
      @glade.get_widget("auto_sleep").text = @vals["auto_sleep"] = val
    when /^@AL,1,2,(\d+)/
      val=$1
      @glade.get_widget("over_speed").text = @vals["over_speed"] = val
    when /^@AL,5,1,(\d+)/
      val=$1
      @glade.get_widget("log_start").text = val
    when /^@AL,5,2,(\d+)/
      val=$1
      @glade.get_widget("log_end").text = val
    when /^@AL,7,1,(.*)/
      val=$1
      @glade.get_widget("dev_name").text = val
    when /^@AL,7,2,(.*)/
      val=$1
      @glade.get_widget("dev_info").text = val
    when /^@AL,8,1,(.*)/
      val=$1
      @glade.get_widget("hw_version").text = val
    when /^@AL,8,2,(.*)/
      val=$1
      @glade.get_widget("sw_version").text = val
    when /^@AL,8,3,(.*)/
      val=$1
      @glade.get_widget("log_version").text = val
    when /^@AL,6,1,(.*)/
      val=$1.to_i
      @glade.get_widget("logcombo").set_active(val)
      @vals['logcombo'] = val
    when /^@AL,6,2,(\d+)/
      val=$1
      @glade.get_widget("low_speed_limit").text = @vals["low_speed_limit"] = val
    when /^@AL,6,3,(\d+)/
      val=$1
      @glade.get_widget("high_speed_limit").text = @vals["high_speed_limit"] = val
    when /^@AL,6,4,(\d+)/
      val=$1
      @glade.get_widget("heading_mode_int").text = @vals["heading_mode_int"] = val
    when /^@AL,6,5,(\d+)/
      val=$1
      @glade.get_widget("low_speed").text = @vals["low_speed"] = val
    when /^@AL,6,6,(\d+)/
      val=$1
      @glade.get_widget("middle_speed").text = @vals["middle_speed"] = val
    when /^@AL,6,7,(\d+)/
      val=$1
      @glade.get_widget("high_speed").text = @vals["high_speed"] = val
    when /^@AL,6,9,(\d+)/
      val=$1
      @glade.get_widget("speed_mode_time").text = @vals["speed_mode_time"] = val
    when /^@AL,6,10,(\d+)/
      val=$1
      @glade.get_widget("low_middle_int").text = @vals["low_middle_int"] = val
    when /^@AL,6,11,(\d+)/
      val=$1
      @glade.get_widget("middle_high_int").text = @vals["middle_high_int"] = val
    when /^@AL,6,12,(\d+)/
      val=$1
      @glade.get_widget("high_high_int").text = @vals["high_high_int"] = val
    when /^@AL,6,13,(\d+)/
      val=$1
      @glade.get_widget("time_mode_int").text = @vals["time_mode_int"] = val
    when /^@AL,6,14,(\d+)/
      val=$1
      @glade.get_widget("dist_mode_int").text = @vals["dist_mode_int"] = val
    end
  end

  def wstate n,s
    @glade.get_widget(n).sensitive = s
    @glade.get_widget(n).editable = s
  end

  def on_log_type_changed(widget)
    state = @glade.get_widget("logcombo").active.to_i
    WMODES.each { |n| wstate(n,false) if n }
    case state
    when 1
      %w(low_speed_limit high_speed_limit heading_mode_int).each {|n| wstate(n,true)}
    when 2
      %w(low_speed_limit high_speed_limit low_speed middle_speed high_speed speed_mode_time
          low_middle_int middle_high_int high_high_int).each {|n| wstate(n,true)}
    when 3
      %w(low_speed_limit high_speed_limit time_mode_int).each {|n| wstate(n,true)}
    when 4
      %w(low_speed_limit high_speed_limit dist_mode_int).each {|n| wstate(n,true)}
    when 5
      %w(low_speed_limit high_speed_limit time_mode_int dist_mode_int).each {|n| wstate(n,true)}
    end
  end

  def on_save_as_kml1_activate(widget)
    puts "on_save_as_kml1_activate() is not implemented yet."
  end

  def is_changed?(w)
    v= nil
    if w
      v0 = @glade.get_widget(w).text
      v1 = @vals[w]
      v = v0 if v0 != v1
    end
    v
  end

  def on_set_clicked(widget)
    WSETS.each_with_index do |w,i|
      if ((v = is_changed?(w)))
	@actions << "1,#{i},#{v}"
      end
    end

    v= @glade.get_widget("logcombo").active
    if v != @vals['logcombo']
      @actions << "6,1,#{v}"
    end

    WMODES.each_with_index do |w,i|
      if ((v = is_changed?(w)))
	@actions << "6,#{i},#{v}"
      end
    end
    kick_actions
  end

  def on_about1_activate(widget)
#    glade.get_widget('About').run
    g = Gtk::AboutDialog.new
    Gtk::AboutDialog.show(nil,
			  {
			    :name=>"WBT-201 viewer",
			    :version=>'07.05.14',
			    :copyright=>'(c) 2007 Jonathan Hudson',
			    :license=>'GNU Public Licence',
			    :website=>'http://daria.co.uk/gps/',
			    :authors=>["Jonathan Hudson <jh+gps@daria.co.uk>"],
			    :documenters=>["you'll be lucky!"]
			  })
  end

  def on_clear_clicked(widget)
    @actions = %w(5,1 5,2)
    @ser.print("@AL,5,6\r\n")
  end

  def on_defaults_clicked(widget)
    puts "on_defaults_clicked() is not implemented yet."
  end

  def on_quit_clicked(widget)
    Gtk.main_quit
  end

  def on_prefs_activate(widget)
    puts "on_prefs_activate() is not implemented yet."
  end

  def on_save_gpx1_activate(widget)
    puts "on_save_gpx1_activate() is not implemented yet."
  end

  def on_window1_delete_event(widget, arg0)
    Gtk.main_quit
  end

  def on_quit1_activate(widget)
    Gtk.main_quit
  end

  def on_notebook_switch_page(widget, arg0, arg1)
    if @constate
    case arg1
    when 0,'0'
      @actions = [nil, nil, nil, nil, nil,'5,2']
    when 1,'1'
      @actions = ['2,1']
    end
    kick_actions
    end
  end

  def on_About_delete_event(widget, arg0)
    widget.hide
  end
  def on_save_gpx1_activate(widget)
    @glade.get_widget("filechooserdialog1").run
  end

end

# Main program
if __FILE__ == $0
  # Set values as your own application. 
  PROG_PATH = "wbt201.glade"
  PROG_NAME = "wbt-201"
  w = Wbt201Glade.new(PROG_PATH, nil, PROG_NAME)
  w.setup
  Gtk.main
end
