/*
 * Copyright (C) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <assert.h>
#include "wbt201.h"

void cleanup(void)
{
    if (wbt->serfd != -1)
    {
        int len;
        gdk_input_remove(wbt->gio);
        do {
            len = write(wbt->serfd, "@AL,2,1", sizeof("@AL,2,1")-1);
        } while (len == -1 && (errno == EAGAIN || errno == EINTR));
        tcdrain(wbt->serfd);
        close(wbt->serfd);
    }
}

void setup_serial(int fd,int baudrate)
{
    struct termios tio;   
    memset (&tio, 0, sizeof(tio));
    tio.c_cflag = CS8 | CLOCAL | CREAD | O_NDELAY | O_NONBLOCK;
    tio.c_iflag = IGNPAR;
    tio.c_oflag = 0;
    tio.c_lflag = 0;
    tio.c_cc[VTIME]    = 0;
    tio.c_cc[VMIN]     = 1;
    switch (baudrate)
    {
        case 4800:   baudrate=B4800; break;              
        case 9600:   baudrate=B9600; break;
        case 19200:  baudrate=B19200; break;
        case 38400:  baudrate=B38400; break;
        case 57600:  baudrate=B57600; break;
        case 115200: baudrate=B115200; break;
        case 230400: baudrate=B230400; break;
    }
    cfsetispeed(&tio,baudrate);
    cfsetospeed(&tio,baudrate);
    tcsetattr(fd,TCSANOW,&tio);
}

int init_serial (char *name, int baudrate)
{
    int fd = open(name, O_RDWR|O_NOCTTY|O_NONBLOCK);
    if (fd != -1)
    {
        setup_serial(fd,baudrate);
    }
    set_menu_states(TRUE);
    return fd;
}

void close_serial(void)
{
    GtkWidget *q;

    set_menu_states(FALSE);
    assert(NULL != (q=glade_xml_get_widget (wbt->xml,"conn_button")));
    gtk_button_set_use_stock(GTK_BUTTON(q),TRUE);
    gtk_button_set_label (GTK_BUTTON(q),GTK_STOCK_CONNECT  );
    assert(NULL != (q = glade_xml_get_widget (wbt->xml,"conn_state")));
    gtk_image_set_from_stock(GTK_IMAGE(q), GTK_STOCK_NO,
                             GTK_ICON_SIZE_MENU);
    gtk_main_iteration_do(FALSE);

    if ( wbt->serfd != -1)
    {
        gdk_input_remove(wbt->gio);
        close(wbt->serfd);
        wbt->serfd = -1;
    }
    wbt->connstat = 0;
}

void read_data(void* data, gint source, GdkInputCondition condition )
{
    static gchar statbuf[1024];
    gchar buf[129];
    int len;
    
    do {
        len = read(wbt->serfd, buf, sizeof(buf)-1);
    } while (len == -1 && (errno == EAGAIN || errno == EINTR));
    
    if(len > 0)
    {
        gchar *sp,*ptr;
        sp = buf;
        buf[len] = 0;
        while((ptr = index(sp, '\n')))
        {
            *ptr = 0;
            strcat(statbuf, sp);
            serial_process(statbuf);
            *statbuf = 0;
            sp = ptr+1;
        }
        if (*sp != '\0')
        {
            strcat(statbuf, sp);
        }
    }
    else 
    {
        close_serial();
    }
}
