/*
 * Copyright (C) 2007 Jonathan Hudson <jh+gps@daria.co.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glade/glade.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include "wbt201.h"
#include <libintl.h>

void set_menu_states(int s)
{
    GtkWidget *w;
    assert(NULL != (w=glade_xml_get_widget (wbt->xml,"save_raw_nmea1")));
    gtk_widget_set_sensitive(w,s);
    assert(NULL != (w=glade_xml_get_widget (wbt->xml,"save_as_kml1")));
    gtk_widget_set_sensitive(w,s);
    assert(NULL != (w=glade_xml_get_widget (wbt->xml,"save_gpx1")));
    gtk_widget_set_sensitive(w,s);
}

G_MODULE_EXPORT gboolean on_draw_expose_event (GtkWidget *w, GdkEventExpose  *event,
                                        gpointer user_data)
{
#ifdef HAVE_CAIRO
    cairo_t * cr = gdk_cairo_create(w->window);

    cairo_rectangle(cr, event->area.x, event->area.y,
                       event->area.width, event->area.height);
    cairo_clip(cr);
    graph_draw(w, cr);
    cairo_destroy(cr);
#endif
    return FALSE;
}
#if 0
static char * combo_to_string(GtkWidget *w, int pos)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    char *sec;
    
    model = gtk_combo_box_get_model (GTK_COMBO_BOX(w));
    gtk_tree_model_iter_nth_child (model, &iter, NULL, pos);
    gtk_tree_model_get (model, &iter, 0, &sec, -1);
    return sec;
}
#endif
G_MODULE_EXPORT void on_quit1_activate (GtkWidget *w, gpointer user_data)
{
    saveprefs();
    gtk_main_quit();
}

G_MODULE_EXPORT void on_dev_type_changed (GtkWidget *w, gpointer user_data) {}

G_MODULE_EXPORT void on_window1_delete_event (GtkWidget *w, gpointer user_data)
{
    gtk_main_quit();
}

G_MODULE_EXPORT void on_dialog1_delete_event (GtkWidget *w, gpointer user_data)
{
    gtk_widget_hide(w);
}

G_MODULE_EXPORT void on_connect_clicked  (GtkWidget *w, gpointer user_data)
{
    char *ser;
    GtkWidget *s,*q;
    
    assert(NULL != (s = glade_xml_get_widget (wbt->xml,"conn_state")));    

    if (wbt->serfd == -1)
    {
        gtk_image_set_from_stock(GTK_IMAGE(s),GTK_STOCK_REDO,
                                 GTK_ICON_SIZE_MENU);
        gtk_main_iteration_do(FALSE);
        ser = gtk_combo_box_get_active_text(GTK_COMBO_BOX(wbt->devcombo));
        wbt->serfd = init_serial(ser, 57600);
        if (wbt->serfd == -1)
        {
            char *m,msg[128];
            m = stpcpy(msg, "opening ");
            strcpy(m, ser);
            message_box(msg);
            gtk_image_set_from_stock(GTK_IMAGE(s),GTK_STOCK_NO,
                                     GTK_ICON_SIZE_MENU);
        }
        else
        {
            if(wbt->lastdev && strcmp(ser,wbt->lastdev) != 0)
            {
                g_free(wbt->lastdev);
                wbt->lastdev = g_strdup(ser);
            }
            
            assert(NULL != (q=glade_xml_get_widget (wbt->xml,"conn_button")));
            gtk_button_set_use_stock(GTK_BUTTON(q),TRUE);
            gtk_button_set_label (GTK_BUTTON(q),GTK_STOCK_DISCONNECT);
            gtk_image_set_from_stock(GTK_IMAGE(s),GTK_STOCK_YES,
                                     GTK_ICON_SIZE_MENU);
            wbt->serq = g_slist_append(wbt->serq,NULL);
            wbt->serq = g_slist_append(wbt->serq,NULL);
            wbt->serq = g_slist_append(wbt->serq,NULL);            
            wbt->serq = g_slist_append(wbt->serq,NULL);
            wbt->serq = g_slist_append(wbt->serq,NULL);
            wbt->gio = gdk_input_add(wbt->serfd, GDK_INPUT_READ, read_data, NULL);
            kick_actions();
        }
        g_free(ser);
    }
    else
    {
        close_serial();
    }
}

G_MODULE_EXPORT void on_notebook_switch_page (GtkWidget *w, GtkNotebookPage *page,
                              guint            page_num,
                              gpointer         user_data)  
{
    switch_page(page_num);
}

G_MODULE_EXPORT void on_clear_clicked (GtkWidget *w, gpointer user_data)
{
     wbt->serq = g_slist_append(wbt->serq,"5,6");
     wbt->serq = g_slist_append(wbt->serq,"5,1");
     wbt->serq = g_slist_append(wbt->serq,"5,2");          
     kick_actions();
}

G_MODULE_EXPORT void on_quit_clicked (GtkWidget *w, gpointer user_data)
{
    saveprefs();
    gtk_main_quit();
}

G_MODULE_EXPORT void on_about1_activate (GtkWidget *w, gpointer user_data)
{
    GtkWidget *q;
    assert(NULL != (q=glade_xml_get_widget (wbt->xml,"about")));
    gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(q),VERSION);
    gtk_widget_show(q);
}


G_MODULE_EXPORT void on_log_type_changed (GtkWidget *w, gpointer user_data)
{
    int val = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
    set_edit_state(val);
}

G_MODULE_EXPORT void on_prefs_activate (GtkWidget *w, gpointer user_data)
{
    GtkWidget *q,*tv,*sb,*kb;
    GtkTextBuffer *tb;
    GtkTextIter is,ie;
    char **d;
    char *text0, *text1;
    int n;
    
    assert(NULL != (q=glade_xml_get_widget (wbt->xml,"dialog1")));
    assert(NULL != (tv=glade_xml_get_widget (wbt->xml,"devnames")));
    assert(NULL != (sb=glade_xml_get_widget (wbt->xml,"rembutton")));
    assert(NULL != (kb=glade_xml_get_widget (wbt->xml,"kmlmarks")));
    
    if(wbt->lastdev)
    {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(sb),1);
    }
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(kb),wbt->trackmarks);    
    tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tv));
    gtk_text_buffer_get_start_iter (tb, &is);
    gtk_text_buffer_get_end_iter (tb, &ie); 
    gtk_text_buffer_delete (tb, &is, &ie);
    for(d = wbt->devs; (d && *d); d++)
    {
        if(**d && !g_ascii_isspace (**d))
        {
            gtk_text_buffer_insert (tb,&ie,*d,-1);
            gtk_text_buffer_insert (tb,&ie,"\n",1);
        }
    }
    gtk_text_buffer_get_start_iter (tb, &is);
    text0 = gtk_text_buffer_get_text(tb,&is,&ie,TRUE);
    gtk_widget_show(q);
    n =  gtk_dialog_run(GTK_DIALOG(q));
    gtk_widget_hide(q);    
    if(n == GTK_RESPONSE_OK)
    {
        wbt->trackmarks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(kb));
        if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(sb)) == TRUE)
        {
            char *ser;
            ser = gtk_combo_box_get_active_text(GTK_COMBO_BOX(wbt->devcombo));
            if(wbt->lastdev)
            {
                if(strcmp(wbt->lastdev,ser) != 0)
                {
                    g_free(wbt->lastdev);
                    wbt->lastdev = g_strdup(ser);
                }
            }
            else
            {
                wbt->lastdev = g_strdup(ser);                
            }
        }
        else
        {
            g_free(wbt->lastdev);
            wbt->lastdev = NULL;
        }
        gtk_text_buffer_get_start_iter (tb, &is);
        gtk_text_buffer_get_end_iter (tb, &ie); 
        text1 = gtk_text_buffer_get_text(tb,&is,&ie,TRUE);
        if(strcmp(text0, text1))
        {
            g_strfreev(wbt->devs);
            wbt->devs = g_strsplit_set(text1," \n\r\t", -1);
            makecombo();
        }
        g_free(text0);
        g_free(text1);
        saveprefs();
    }
}

static void stamp_fn(char *basefn, size_t basesz, char *fmt)
{
    struct tm tm;
    time_t t;
    t = time(NULL);
    localtime_r(&t, &tm);
    strftime(basefn, basesz, fmt, &tm);
}

G_MODULE_EXPORT void on_save_raw_nmea1_activate (GtkWidget *w, gpointer user_data)
{
    if(wbt->nmea_fp)
    {
        fclose(wbt->nmea_fp);
        wbt->nmea_fp = NULL;
    }
    else if (GTK_CHECK_MENU_ITEM(w)->active)
    {
        int del=WBT_SAVE_NMEA;
        char *fn;
        char basefn[128];

        stamp_fn(basefn, sizeof(basefn), "nmea_%FT%H.%M.%S.txt");

        fn = save_dialogue(basefn, WBT_FILTER_NONE, &del);
        if(fn)
        {
            wbt->nmea_fp = fopen(fn,(del) ? "a" : "w");
        }
        else
        {
            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w),FALSE);
        }
    }
}

G_MODULE_EXPORT void  on_set_clicked (GtkWidget *w, gpointer user_data)
{
    do_set_actions();
}

G_MODULE_EXPORT void on_defaults_clicked (GtkWidget *w, gpointer user_data)
{
    set_factory_defaults();
}

G_MODULE_EXPORT void on_save_gpx1_activate (GtkWidget *w, gpointer user_data)
{
    int del=(WBT_SAVE_DEL|WBT_SAVE_TRK|WBT_SAVE_WAY);
    char *fn;
    char basefn[128];

    stamp_fn(basefn, sizeof(basefn), "track_%FT%H.%M.%S.gpx");
    
    fn = save_dialogue(basefn, WBT_FILTER_GPX, &del);

    if(fn)
    {
        if (del & (WBT_SAVE_DEL|WBT_SAVE_TRK|WBT_SAVE_WAY))
            babel_spawn(fn, "gpx",del);
        g_free(fn);
    }
}

G_MODULE_EXPORT void on_save_as_kml1_activate (GtkWidget *w, gpointer user_data)
{
    int del=(WBT_SAVE_DEL|WBT_SAVE_TRK|WBT_SAVE_WAY);
    char *fn;
    char basefn[128];

    stamp_fn(basefn, sizeof(basefn), "track_%FT%H.%M.%S.kml");
    fn = save_dialogue(basefn, WBT_FILTER_GPX, &del);
    if(fn)
    {
        if (del & (WBT_SAVE_DEL|WBT_SAVE_TRK|WBT_SAVE_WAY))
        {
            char cmd[32] = "kml";
            if(wbt->trackmarks == FALSE)
            {
                strcat(cmd,",points=0");
            }
	    babel_spawn(fn, cmd, del);
        }
        g_free(fn);
    }
}

G_MODULE_EXPORT void on_about_close (GtkWidget *w, gpointer user_data)
{
    gtk_widget_hide(w);
}


